import { FlowNode } from '../../../Nodes/FlowNode.js';
import { NodeDescription } from '../../../Nodes/Registry/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class Gate extends FlowNode {
    constructor(description, graph) {
        super(description, graph, [
            new Socket('flow', 'flow'),
            new Socket('flow', 'open'),
            new Socket('flow', 'close'),
            new Socket('flow', 'toggle'),
            new Socket('boolean', 'startClosed', true)
        ], [new Socket('flow', 'flow')]);
        this.isInitialized = false;
        this.isClosed = true;
    }
    triggered(fiber, triggeringSocketName) {
        if (!this.isInitialized) {
            this.isClosed = this.readInput('startClosed');
            this.isInitialized = true;
        }
        switch (triggeringSocketName) {
            case 'flow': {
                if (!this.isClosed) {
                    fiber.commit(this, 'flow');
                }
                break;
            }
            case 'open': {
                this.isClosed = false;
                return;
            }
            case 'close': {
                this.isClosed = true;
                return;
            }
            case 'toggle': {
                this.isClosed = !this.isClosed;
                return;
            }
            default:
                new Error('should not get here');
        }
    }
}
Gate.Description = new NodeDescription('flow/gate', 'Flow', 'Gate', (description, graph) => new Gate(description, graph));
//# sourceMappingURL=Gate.js.map