import { FlowNode } from '../../../Nodes/FlowNode.js';
import { NodeDescription } from '../../../Nodes/Registry/NodeDescription.js';
import { Socket } from '../../../Sockets/Socket.js';
export class MultiGate extends FlowNode {
    constructor(description, graph) {
        super(description, graph, [
            new Socket('flow', 'flow'),
            new Socket('flow', 'reset'),
            new Socket('boolean', 'loop', true),
            new Socket('integer', 'startIndex', 0)
        ], [
            new Socket('flow', '1'),
            new Socket('flow', '2'),
            new Socket('flow', '3')
        ]);
        this.isInitialized = false;
        this.nextIndex = 0;
    }
    triggered(fiber, triggeringSocketName) {
        if (!this.isInitialized) {
            this.nextIndex = Number(this.readInput('startIndex'));
        }
        if (this.readInput('loop')) {
            this.nextIndex = this.nextIndex % this.outputSockets.length;
        }
        switch (triggeringSocketName) {
            case 'reset': {
                this.nextIndex = 0;
                return;
            }
            case 'flow': {
                if (0 <= this.nextIndex && this.nextIndex < this.outputSockets.length) {
                    fiber.commit(this, this.outputSockets[this.nextIndex].name);
                }
                this.nextIndex++;
                return;
            }
        }
        const sequenceIteration = (i) => {
            if (i < this.outputSockets.length) {
                const outputSocket = this.outputSockets[i];
                fiber.commit(this, outputSocket.name, () => {
                    sequenceIteration(i + 1);
                });
            }
        };
        sequenceIteration(0);
    }
}
MultiGate.Description = new NodeDescription('flow/multiGate', 'Flow', 'MultiGate', (description, graph) => new MultiGate(description, graph));
//# sourceMappingURL=MultieGate.js.map